#ifndef _COMMON_H
#define _COMMON_H

/* comment this if you don't want debugging */
#define DEBUG

/* settings of real modem */
#define REAL_MODEM_TTY      "ttyS2"
#define REAL_MODEM_BAUDRATE B115200

#define SERIAL_SUFFIX "z0"

#define FIFO_SIZE 4096
#define FIFO_INDEX(x) ((x) & (FIFO_SIZE - 1))

#define SAMPLE_RATE  (28800)
#define SAMPLE_SIZE  sizeof(short)
/* OSS buffer size */
#define FRAGMENT_BITS 10
#define NB_FRAGMENTS  (32768 >> FRAGMENT_BITS)
/* 1024 samples : 36 ms delay - will be good enougth for real-time duplex */
#define NB_SAMPLES ((1 << FRAGMENT_BITS)/SAMPLE_SIZE)

#define BAUDRATE     (1200)

#define BAUD_TIME    (SAMPLE_RATE/BAUDRATE)
#define START        (int)((double)BAUD_TIME * 0.2833)
#define STOP         (int)((double)BAUD_TIME * 0.65)
#define HOLD_DELAY   (SAMPLE_RATE * 5 - NB_SAMPLES)             /* 5 seconds */
#define RR_DELAY     ((int)((double)SAMPLE_RATE * 0.0225) - NB_SAMPLES) /* 22.5 ms */
#define NC_TIMEOUT   (SAMPLE_RATE * 20)   /* 20 seconds NO CARRIER timeout */

#define FILTER_ORDER         15 /* Low pass filter for discriminator output */

#define BIT_SYNC             0x5555UL    /* bit syncronization sequence */
#define BIT_SYNC_MASK        (1UL << 14)
#define FRAME_SYNC           0x147UL     /* frame syncronization word */
#define FRAME_SYNC_MASK      (1UL << 10)
#define DATA_MASK            (1UL << 8)

#define RX_STATUS_READY      0 /* got bit syncronization sequence */
#define RX_STATUS_SYNC_OK    1 /* got frame syncronization word */
#define RX_STATUS_OK         2 /* successfully received frame */
#define RX_STATUS_BAD_CRC    3 /* recived frame have bad CRC */
#define RX_STATUS_IGNORE     4 /* frame with wrong direction bit received */

#define TX_STATUS_SILENCE    0 /* transmitting nothing */
#define TX_STATUS_BIT_SYNC   1 /* transmitting bit syncronization sequence */
#define TX_STATUS_FRAME_SYNC 2 /* transmitting frame syncronization word */
#define TX_STATUS_DATA       3 /* transmitting DT/CT frame */
#define TX_STATUS_ACK        4 /* transmitting RR/NR frame */

#define DT_PREFIX            0x18
#define RR_PREFIX            0x08
#define NR_PREFIX            0x10

#define NEGATIVE_RESPONSE    0x20
#define POSITIVE_RESPONSE    0x00

#define CRC_POLYNOMINAL      0x11021

#include "transmitter.h"
#include "receiver.h"

struct _dms_modem
{
  struct _transmitter tx;
  struct _receiver    rx;
/* DSP stuff */
  int                 dsp_fd;              /* DSP file descriptor */
  short               dsp_buf[NB_SAMPLES]; /* buffer */
/* pty stuff */
  int                 pty_fd;
};

#endif