/* See COPYING for license details */
/* (C)2000 Andrew Kozin AKA Stanson */

#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <limits.h>
#include <signal.h>
#include <math.h>
#include <termios.h>
#include <sys/wait.h>
#include <sys/time.h>
#include <sys/soundcard.h>
#include <sys/ioctl.h>

#include "common.h"
#include "real_modem.h"

char  *pty_name = "/dev/pty"SERIAL_SUFFIX;
char  *tty_name = "/dev/tty"SERIAL_SUFFIX;
char  *dsp_name = "/dev/dsp";

int   debug = 0, modem_fd;

int   old_mixer[SOUND_MIXER_NRDEVICES];
int   old_recsrc;
int   old_outsrc;

struct     _dms_modem mdm;

#define VOLUME(x) ((x)|((x)<<8))

int dsp_ioctl(int fd, int req, int param)
{
  return ioctl(fd,req,&param);
}

int dsp_open(char *name)
{int          i, fd;
 unsigned int devmask;

  if((fd = open(dsp_name, O_RDWR)) < 0)
  {
    fprintf(stderr, "Can't open %s!\n",name);
    return fd;
  }

  if(ioctl(fd, SNDCTL_DSP_SETDUPLEX) == -1) goto err;
  if(dsp_ioctl(fd, SNDCTL_DSP_SETFRAGMENT, (NB_FRAGMENTS << 16) | FRAGMENT_BITS) == -1) goto err;
  if(dsp_ioctl(fd, SNDCTL_DSP_SETFMT,      AFMT_S16_LE) == -1) goto err;
  if(dsp_ioctl(fd, SNDCTL_DSP_STEREO,      0) == -1) goto err;
  if(dsp_ioctl(fd, SNDCTL_DSP_SPEED,       SAMPLE_RATE) == -1) goto err;

  if(ioctl(fd, MIXER_READ(SOUND_MIXER_RECSRC), &old_recsrc) == -1) goto err;
  if(ioctl(fd, MIXER_READ(SOUND_MIXER_OUTSRC), &old_outsrc) == -1) goto err;
  if(ioctl(fd, MIXER_READ(SOUND_MIXER_DEVMASK), &devmask) == -1) goto err;
  for(i = 0; i < SOUND_MIXER_NRDEVICES; i++, devmask >>= 1)
  {
    if(devmask & 1)
    {
      if(ioctl(fd, MIXER_READ(i), &old_mixer[i]) == -1) goto err;
      if(dsp_ioctl(fd, MIXER_WRITE(i), 0) == -1) goto err;
    }
  }
  if(dsp_ioctl(fd, MIXER_WRITE(SOUND_MIXER_OUTSRC), SOUND_MASK_PCM) == -1) goto err;
  if(dsp_ioctl(fd, MIXER_WRITE(SOUND_MIXER_RECSRC), SOUND_MASK_LINE) == -1) goto err;
  if(dsp_ioctl(fd, MIXER_WRITE(SOUND_MIXER_PCM),    VOLUME(100)) == -1) goto err;
  if(dsp_ioctl(fd, MIXER_WRITE(SOUND_MIXER_VOLUME), VOLUME(100)) == -1) goto err;
  if(dsp_ioctl(fd, MIXER_WRITE(SOUND_MIXER_LINE),   VOLUME(90)) == -1) goto err;
  return fd;

err:
  close(fd);
  fprintf(stderr, "Can't ioctl %s!\n", name);
  return -1;
}

void dsp_close(int fd)
{int          i;
 unsigned int devmask;

  if(ioctl(fd, MIXER_READ(SOUND_MIXER_DEVMASK), &devmask) != -1)
  {
    for(i = 0; i < SOUND_MIXER_NRDEVICES; i++, devmask >>= 1)
    {
      if(devmask & 1) ioctl(fd, MIXER_WRITE(i), &old_mixer[i]);
    }
  }
  ioctl(fd, MIXER_WRITE(SOUND_MIXER_RECSRC), &old_recsrc);
  ioctl(fd, MIXER_WRITE(SOUND_MIXER_OUTSRC), &old_outsrc);
  close(fd);
}

void term_handler( int sig )
{
  fprintf(stderr, "Got signal %d, exiting...\n",sig);
  dsp_close(mdm.dsp_fd);
  close(mdm.pty_fd);
  modem_close(modem_fd);
  exit(0);
}

int main(int argc, char *argv[])
{int               ret;
 struct sigaction  act;
 struct termios    ti;

  /* first of all set SIGTERM signal handler */
  act.sa_handler = term_handler;
  act.sa_flags   = 0;
  sigaction(SIGTERM, &act, NULL);
  sigaction(SIGINT, &act, NULL);

  /* initialize mdm */
  memset(&mdm,0,sizeof(mdm));

  if((modem_fd = modem_open()) < 0)
  {
    fprintf(stderr, "Can't open modem !\n");
    return 1;
  }

  printf("dms_modem tty is '%s'\n", tty_name);

  if((mdm.pty_fd = open(pty_name, O_RDWR | O_NONBLOCK)) < 0)
  {
    fprintf(stderr, "Can't open %s !\n", pty_name);
    return 1;
  }
  ti.c_cflag = B115200 | CS8 | CLOCAL | CREAD;
  ti.c_iflag = 0;
  ti.c_lflag = 0;
  ti.c_oflag = 0;
  ti.c_cc[VMIN] = 1;
  ti.c_cc[VTIME] = 0;
  tcsetattr(mdm.pty_fd, TCSANOW, &ti);

  printf("dms_modem dsp is '%s'\n", dsp_name);

  if((mdm.dsp_fd = dsp_open(dsp_name)) < 0)
  {
    close(mdm.pty_fd);
    return 1;
  }

  if(argc > 1)
  {
    if((debug = open(argv[1], O_RDWR | O_CREAT | O_TRUNC, 0664)) < 0)
    {
      fprintf(stderr, "Can't open %s !\n", argv[1]);
      return 1;
    }
  }

  tx_init(&mdm.tx);
  rx_init(&mdm.rx, &mdm.tx);

  for(;;)
  {
    ret = tunnel(&mdm, modem_fd);

    ret = read(mdm.dsp_fd, mdm.dsp_buf, sizeof(mdm.dsp_buf));
    if(ret != sizeof(mdm.dsp_buf))
    {
      fprintf(stderr, "Error reading %s!\n",dsp_name);
      goto out;
    }
    if(debug) write(debug, mdm.dsp_buf, sizeof(mdm.dsp_buf));

    rx_decode(&mdm.rx,mdm.dsp_buf,NB_SAMPLES);
    tx_encode(&mdm.tx,mdm.dsp_buf,NB_SAMPLES);

    ret = write(mdm.dsp_fd, mdm.dsp_buf, sizeof(mdm.dsp_buf));
    if(ret != sizeof(mdm.dsp_buf))
    {
      fprintf(stderr, "Error reading %s!\n",dsp_name);
      goto out;
    }
  }
out:
  dsp_close(mdm.dsp_fd);
  close(mdm.pty_fd);
  return 1;
}

