#include <sys/types.h>
#include <sys/stat.h>
#include <signal.h>
#include <fcntl.h>
#include <termios.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <ctype.h>

#include "common.h"
#include "real_modem.h"

static struct termios oldtio;

int tty_lock(char *lckname)
{FILE *f;pid_t pid;
 char tmpname[256], *p;
 int  rc;

  f=fopen(lckname, "rt");
  if(f)
  {
    fscanf(f, "%d", &pid);
    fclose(f);
    if(kill(pid, 0) && (errno == ESRCH))
    {
      printf("Lockfile is stale, overriding it.\n");
      unlink(lckname);
    }
    else return 0;
  }

  strcpy(tmpname, lckname);
  p=strrchr(tmpname, '/');if(!p) p=tmpname;
  sprintf(p, "/DMS_LCK.%d", getpid());
  if ((f=fopen(tmpname,"w")) == NULL) return 0;
  fprintf(f,"%10d\n",getpid());
  fclose(f);
  unlink(lckname);
  rc=link(tmpname,lckname);
  if(rc) return 0;
  unlink(tmpname);
  return 1;
}

void tty_unlock(char *lckname)
{int pid;
 FILE *f;
	
  if ((f=fopen(lckname,"r")))
  {
    fscanf(f,"%d",&pid);
    fclose(f);
  }
  if(pid==getpid()) unlink(lckname);
}

int modem_open(void)
{int    modem_fd = -1;
 struct termios newtio;

  if(!tty_lock("/var/lock/LCK.."REAL_MODEM_TTY))
  {
    printf("Serial port "REAL_MODEM_TTY" already locked\n");
    return -1;
  }
  if((modem_fd = open("/dev/"REAL_MODEM_TTY, O_RDWR | O_NOCTTY | O_NONBLOCK)) < 0)
  {
    tty_unlock("/var/lock/LCK.."REAL_MODEM_TTY);
    return -1;
  }
  tcgetattr(modem_fd,&oldtio);
  tcflush(modem_fd, TCIFLUSH);
  memset(&newtio, 0, sizeof(newtio));
  newtio.c_cflag     = REAL_MODEM_BAUDRATE | CRTSCTS | CS8 | CLOCAL | CREAD;
  newtio.c_iflag     = IGNPAR;
  newtio.c_oflag     = 0;
  newtio.c_lflag     = 0;
  newtio.c_cc[VTIME] = 0;
  newtio.c_cc[VMIN]  = 1;
  tcsetattr(modem_fd,TCSANOW,&newtio);
  return modem_fd;
}

void modem_close(int modem_fd)
{
  tcsetattr(modem_fd,TCSANOW,&oldtio);
  close(modem_fd);
  tty_unlock("/var/lock/LCK.."REAL_MODEM_TTY);
}

int tunnel(struct _dms_modem *m, int modem_fd)
{int  ret = 0;
 char c;
 static char in_buf[256],
             cmd[256],
             *at = "AT";
 static int  in_len = 0, 
             cmd_len = 0,
             att_timeout = 0,
             plus = 0,
             command_mode = 1;

  /* passthrough input */
  if(command_mode)
  {/* take data from real modem */
    ret = read(modem_fd, in_buf, sizeof(in_buf));
    if(ret > 0)
    {
      write(m->pty_fd, in_buf, ret);
    }
  }
  else if(m->rx.tty_len)
  {/* take data for output from DMS modem output buffer */
    ret = write(m->pty_fd, m->rx.tty_buf, m->rx.tty_len);
    if(ret > 0)
    {
      m->rx.tty_len -= ret;
      memmove(m->rx.tty_buf, &m->rx.tty_buf[ret], m->rx.tty_len);
    }
  }

  /* passthrough output, scan it for ATA and +++ATH0 and set apropriate state */
  if(m->tx.tty_len < sizeof(m->tx.tty_buf))
  {
    att_timeout = 1;
    while(((ret = read(m->pty_fd, &c, sizeof(c))) > 0) &&
          (m->tx.tty_len < sizeof(m->tx.tty_buf)))
    {
      if(command_mode)
      {
        if(*at)
        {
          write(modem_fd, &c, sizeof(c));
          if(toupper(c) != *at++) at = "AT";
        }
        else
        {
          cmd[cmd_len++] = toupper(c);
          switch(cmd[0])
          {
            case 'A':
              if(cmd_len == 1) {write(modem_fd, "H", 1); c = '1';}
              if((c == '\n') || (c == '\r'))
              {
                command_mode = 0;
                in_len = 0;
              }
              break;
            case 'H':
              if(cmd[1] != '0') break;
            case 'Z':
              if((c == '\n') || (c == '\r'))
              {/* reset DMS modem */
                tx_init(&m->tx);
                rx_init(&m->rx, &m->tx);
              }
              break;
          }
          write(modem_fd, &c, sizeof(c));
          if((c == '\n') || (c == '\r'))
          {
            cmd[cmd_len] = 0;
            printf("Command: AT%s\n", cmd);
            cmd_len = 0;
            at = "AT";
            break;
          }
        }
      }
      else
      {
        m->tx.tty_buf[m->tx.tty_len++] = c;
        if((att_timeout || plus) && (c == '+'))
        {
          plus++;
        }
        else
        {
          plus = 0;
        }
      }
      att_timeout = 0;
    }
    if(att_timeout && (plus == 3))
    {
      command_mode = 1;
      plus = 0;
      tcflush(modem_fd,TCIOFLUSH);
    }
  }
  return 0;
}

