/* See COPYING for license details
 * (C)2000 Andrew Kozin AKA Stanson */
#include <string.h>
#include <stdio.h>
#include <math.h>
#include <limits.h>

#include "common.h"
#include "real_modem.h"

/*
 * coefficients for bandpass filter
 */
long bandpass[]=
{
    7090,   -6707,   -1879,   -2072,   -3679,   -4102,   -5253,   -4550,
   -4444,   -2420,   -1086,    1749,    3724,    6347,    7862,    8957,
    8957,    7862,    6347,    3724,    1749,   -1086,   -2420,   -4444,
   -4550,   -5253,   -4102,   -3679,   -2072,   -1879,   -6707,    7090
};

short FIR_process(struct _FIR *fir, short input)
{long sum = 0;
 int  i,j = fir->order;

  /* store input sample to buffer */
  i = fir->idx++;
  fir->buf[FIR_BUF_INDEX(i)] = input;
  /* process FIR */
  while(j--)
  {
    sum += (long)fir->buf[FIR_BUF_INDEX(i - j)] * fir->weights[j];
  }
  sum >>= 16;
  return (short)sum;
}

void rx_bit(struct _receiver *rx, int bit)
{int             i;
 unsigned long   c;
 unsigned short  cs;

  rx->stream <<= 1;
  rx->stream |= bit;

  if((rx->stream & 0x7FFUL) == 0x555UL)
  {/* Got sync sequence - new frame sequence */
    rx->status  = RX_STATUS_READY;
    rx->frm_cnt = 0;
    return;
  }

  if((rx->stream & 0x7FFUL) == FRAME_SYNC)
  {/* We get synchronization word, so the frame began */
    rx->status   = RX_STATUS_SYNC_OK;
    rx->bit_num  = 2;
    rx->word_cnt = 0;
    rx->bit_cnt  = 6;
    rx->frm_cnt++;
    return;
  }

  if(rx->status == RX_STATUS_SYNC_OK)
  {
    if(!rx->bit_cnt)
    {
      rx->bit_cnt = 8;
      rx->data[rx->word_cnt++] = rx->stream & 0x7F;
    }
    else
    {
      rx->bit_cnt--;
    }

    if(rx->bit_num == 21)
    {
      if((rx->frm_cnt > 1) && (rx->data[0] == rx->data[1]))
      {
        switch(rx->data[0] & 0x18)
        {
          case RR_PREFIX:
#ifdef DEBUG
            printf(">RR(%u)\n", rx->data[0] & 7);
#endif
            rx->tx->rr_label = rx->data[0];
            rx->status = RX_STATUS_OK;
            break;
          case NR_PREFIX:
#ifdef DEBUG
            printf(">NR(%u)\n", rx->data[0] & 7);
#endif
            rx->tx->rr_label = rx->data[0];
            rx->status = RX_STATUS_OK;
            break;
        }
      }
    }
    else if(rx->bit_num == 102)
    {
      rx->data[rx->word_cnt] = (rx->stream << 5) & 0x7F;
      cs = get_checksum(&rx->data[0]);

      if(!cs)
      {/* CRC OK */
        rx->status = RX_STATUS_OK;
        if((rx->data[0] & 0x18) == DT_PREFIX)
        {/* DT/CT frame received */
          if((rx->data[0] & 7) == rx->nrx)
          {
#ifdef DEBUG
            printf(">%cT(%u) ", rx->data[0] & 0x20 ? 'D' : 'C', rx->data[0] & 7);
            for(i = 1; i < 9; i++) printf("%02X ", rx->data[i]);
            printf("\n");
#endif
            rx->tx->dt_label = rx->data[0];

            /* Save DT/CT(NRX) */
            if(rx->data[0] & 0x20)
            {/* Data frame */
              if(rx->tx->wif == 7)
              {/* convert data to 8-bit */
                for(i = 2, c = rx->data[1]; i < 9; i++) { c <<= 1; rx->data[i] |= c & 0x80; }
                /* strip NUL characters from tail */
                for(i = 8; (i > 2) && (!rx->data[i]); i--);
                /* put data to output buffer */
                memcpy(&rx->tty_buf[rx->tty_len], &rx->data[2], i - 1);
                rx->tty_len += i - 1;
              }
              else
              {
                /* strip NUL characters from tail */
                for(i = 8; (i > 1) && (!rx->data[i]); i--);
                /* put data to output buffer */
                memcpy(&rx->tty_buf[rx->tty_len], &rx->data[1], i);
                rx->tty_len += i;
              }
            }
            else
            {/* Control frame */
              switch(rx->data[1])
              {
                case 0:   /* IDLE */
                  break;
                case 'I': /* ID */
                  memcpy(rx->remote_id,&rx->data[2],7);
                  break;
                case 'R': /* RAND */
                  /* we must reply with our RAND */
                  rx->tx->control[0]   = 'R';
                  rx->tx->control[1]   = 0;
                  rx->tx->control[2]   = 0;
                  rx->tx->control[3]   = 0;
                  rx->tx->control[4]   = 0;
                  rx->tx->control[5]   = 0;
                  rx->tx->control[6]   = 0;
                  rx->tx->control[7]   = rx->data[7];
                  rx->tx->control_flag = 1;
                  rx->tx->dir          = (rx->data[0] & 0x40) ^ 0x40;
                  rx->tx->wif          = (rx->data[8] == '8') ? 7 : 8;
                  memcpy(&rx->tty_buf[rx->tty_len], "CONNECT 1200/DMS RID:", 21);
                  rx->tty_len += 21;
                  memcpy(&rx->tty_buf[rx->tty_len], rx->remote_id, 7);
                  rx->tty_len += 7;
                  rx->tty_buf[rx->tty_len++] = '\r';
                  rx->tty_buf[rx->tty_len++] = '\n';
                  break;
                case 'T': /* A-NUM */
                  break;
                case '0': /* HANGUP48 */
                  break;
                case 'N': /* TRAN */
                  break;
              }
            }
            /* Increase NRX */
            rx->nrx = ((rx->data[0] & 7) + 1) & 7;
          }
        }
      }
      else
      {/* Bad CRC */
        rx->status = RX_STATUS_BAD_CRC;
#ifdef DEBUG
        printf(">%cT(%u) ", rx->data[0] & 0x20 ? 'D' : 'C', rx->data[0] & 7);
        for(i = 1; i < 9; i++) printf("%02X ", rx->data[i]);
        printf("CS:%04X", cs);
        printf(" Bad CRC!\n");
#endif
      }
    }
    rx->bit_num++;
  }
}

int rx_decode(struct _receiver *rx, short *buf, int size)
{int i, data;

  for(i = 0; i < size; i++, buf++, rx->ci++)
  {
    /* input bandpass filter */
    *buf = FIR_process(&rx->fir, *buf);

    rx->ci &= 63;
    /* Zero crossing detector */
    if((rx->os & 0x8000) != (*buf & 0x8000))
    {
      /* Start monostable */
      rx->timer = BAUD_TIME / 3;
      /* Bad zero crossing detector */
      if((rx->counter < START) || (rx->counter > STOP))
      {
        rx->no_carrier = BAUD_TIME * 2;
      }
      rx->counter = 0;
    }
    rx->os = *buf;
    /* Monostable */
    if( rx->timer > 0 ) { rx->monostable[rx->ci] = 1; rx->timer--; }
    else                { rx->monostable[rx->ci] = 0; }
    /* Discriminator */
    rx->disc[rx->ci] = rx->monostable[rx->ci] &
                   rx->monostable[(rx->ci - BAUD_TIME/6) & 63] &
                   rx->monostable[(rx->ci - BAUD_TIME/3) & 63];
    /* Finite impulse response filter */
    rx->disc_FIR += (rx->disc[rx->ci] - rx->disc[(rx->ci - FILTER_ORDER + 1) & 63]) * 127 / FILTER_ORDER;
    /* Bad zero crossing detector */
    if(rx->counter++ > STOP)
    {
      rx->no_carrier = BAUD_TIME * 2;
      rx->counter = 0;
    }
    /* Carrier detector */
    if(rx->no_carrier)
    {
      rx->no_carrier--;
    }
    else
    {
      data = rx->disc_FIR < 64 ? 1 : 0;
      /* First order digital PLL */
      if((rx->old_data != data) || !rx->pll--) { rx->pll = BAUD_TIME; }
      rx->old_data = data;
      if(rx->pll == BAUD_TIME/2) rx_bit(rx, data);
    }
  }
  return 0;
}

int rx_init(struct _receiver *rx, struct _transmitter *tx)
{
  memset(rx,0,sizeof(struct _receiver));
  rx->tx = tx;
  rx->no_carrier  = BAUD_TIME * 2;
  rx->fir.order   = 32;
  rx->fir.weights = bandpass;
  return 0;
}
